@@ -1,7 +1,7 @@
-# Copyright (c) 2004 CentralNic Ltd. All rights reserved. This program is
+# Copyright (c) 2007 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
-# $Id: CNic.pm,v 1.40 2006/05/02 15:57:01 gavin Exp $
+# $Id: CNic.pm,v 1.45 2007/04/26 16:16:07 gavin Exp $
 package WWW::CNic;
 use LWP;
 use LWP::ConnCache;
@@ -11,7 +11,7 @@ use Digest::MD5 qw(md5_hex);
 use vars qw($VERSION $CONNECTION_CACHE);
 use strict;
 
-our $VERSION = '0.18';
+our $VERSION = '0.22';
 
 =pod
 
@@ -223,6 +223,10 @@ sub execute {
 		$self->{_command} eq 'cancel_transfer'	&& return $self->_cancel_transfer();
 		$self->{_command} eq 'reactivate'	&& return $self->_reactivate_domain();
 		$self->{_command} eq 'push_domain'	&& return $self->_push_domain();
+		$self->{_command} eq 'auth_info'	&& return $self->_auth_info();
+		$self->{_command} eq 'poll_transfers'	&& return $self->_poll_transfers();
+		$self->{_command} eq 'approve_transfer'	&& return $self->_approve_transfer();
+		$self->{_command} eq 'reject_transfer'	&& return $self->_reject_transfer();
 		die("Invalid command '$self->{_command}'");
 	}
 }
@@ -257,14 +261,31 @@ sub _suffixes {
 sub _create_handle {
 	my $self = shift;
 	die("Invalid type for handle") unless (ref($self->{_params}->{handle}) eq 'HASH');
-	my $handle = $self->_build_handle_string($self->{_params}->{handle});
 	my %params =  (
 		user		=> $self->{_username},
 		password	=> $self->_crypt_md5($self->{_password}),
 		test		=> $self->{_test},
-		handle		=> $handle,
 		visible		=> ($self->{_params}->{visible} == 0 ? 0 : 1),
 	);
+	if (defined($self->{_params}->{handle}->{street1})) {
+		$params{name}		= $self->{_params}->{handle}->{name};
+		$params{company}	= $self->{_params}->{handle}->{company};
+		$params{street1}	= $self->{_params}->{handle}->{street1};
+		$params{street2}	= $self->{_params}->{handle}->{street2};
+		$params{street3}	= $self->{_params}->{handle}->{street3};
+		$params{city}		= $self->{_params}->{handle}->{city};
+		$params{sp}		= $self->{_params}->{handle}->{sp};
+		$params{postcode}	= $self->{_params}->{handle}->{postcode};
+		$params{country}	= $self->{_params}->{handle}->{country};
+		$params{phone}		= $self->{_params}->{handle}->{phone};
+		$params{fax}		= $self->{_params}->{handle}->{fax};
+		$params{email}		= $self->{_params}->{handle}->{email};
+
+	} else {
+		$params{handle} = $self->_build_handle_string($self->{_params}->{handle});
+
+	}
+
 	my $req = HTTP::Request->new(POST => "$self->{_base}/create_handle");
 	$req->content_type('application/x-www-form-urlencoded');
 	my @content;
@@ -688,6 +709,76 @@ sub _push_domain {
 	return WWW::CNic::Response::PushDomain->new($self->{_response}->{_raw});
 }
 
+sub _auth_info {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+				domain		=> $self->{_domain},
+	);
+	my $url = "$self->{_base}/auth_info";
+	my $req = POST($url, \%params);
+	$self->{_response}->{_raw} = $self->_get($req);
+	use WWW::CNic::Response::AuthInfo;
+	return WWW::CNic::Response::AuthInfo->new($self->{_response}->{_raw});
+}
+
+sub _poll_transfers {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+	);
+	my $url = "$self->{_base}/poll_transfers";
+	my $req = POST($url, \%params);
+	$self->{_response}->{_raw} = $self->_get($req);
+	use WWW::CNic::Response::PollTransfers;
+	return WWW::CNic::Response::PollTransfers->new($self->{_response}->{_raw});
+}
+
+sub _approve_transfer {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+				domain		=> $self->{_domain},
+	);
+	my $url = "$self->{_base}/approve_transfer";
+	my $req = POST($url, \%params);
+	$self->{_response}->{_raw} = $self->_get($req);
+	use WWW::CNic::Response::ApproveTransfer;
+	return WWW::CNic::Response::ApproveTransfer->new($self->{_response}->{_raw});
+}
+
+sub _reject_transfer {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+				domain		=> $self->{_domain},
+	);
+	my $url = "$self->{_base}/reject_transfer";
+	my $req = POST($url, \%params);
+	$self->{_response}->{_raw} = $self->_get($req);
+	use WWW::CNic::Response::RejectTransfer;
+	return WWW::CNic::Response::RejectTransfer->new($self->{_response}->{_raw});
+}
+
 sub _get {
 	my ($self, $request) = @_;
 	my $response = $self->{_agent}->request($request);
@@ -713,9 +804,16 @@ sub _salt {
 
 sub _crypt_md5 {
 	my ($self, $str) = @_;
-	my $salt = '$1$' . substr(md5_hex(rand()), 0, 8) . '$';
-	my $crypted = crypt($str, $salt);
-	return $crypted;
+	### special hack: Mac OS X doesn't support MD5 in the crypt() function
+	### so use plaintext passwords:
+	if ($^O eq 'darwin') {
+		return 'plain:'.$str;
+
+	} else {
+		return crypt($str, '$1$' . substr(md5_hex(rand()), 0, 8) . '$');
+
+	}
+
 }
 
 1;
