@@ -1,7 +1,7 @@
-# Copyright (c) 2006 CentralNic Ltd. All rights reserved. This program is
+# Copyright (c) 2007 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
-# $Id: Cookbook.pm,v 1.35 2006/05/02 15:57:01 gavin Exp $
+# $Id: Cookbook.pm,v 1.39 2007/04/26 16:16:07 gavin Exp $
 
 package WWW::CNic::Cookbook;
 use vars qw($VERSION);
@@ -141,6 +141,11 @@ The C<registrant> and C<status> keys are just strings. The C<created> and C<expi
 	name				
 	company
 	addr
+	street1
+	street2
+	street3
+	city
+	sp
 	postcode
 	country
 	phone
@@ -158,6 +163,32 @@ These values can be accessed in the following way:
 
 The C<dns> key is an array. Elements in the array may be either a plain scalar containing the DNS hostname of the server, or an anonymous array containing the DNS hostname and IPV4 address in that order. Use the C<ref()> function to work out which.
 
+B<Address data:> handles in the CentralNic database follow one of two "styles": old-style handles have flat address data (stored in the C<address> property), but new-style handles have separate properties for street, city and state/province. All handles will have at least a non-empty C<address> property (for new-style handles this is created by flattening the C<street1>, C<street2>... properties). All new-style handles will have a non-empty C<street1> property.
+
+=head2 Retrieving the Transfer Code For a Domain Name
+
+	use WWW::CNic;
+
+	my $query = WWW::CNic->new(
+		use_ssl		=> 1,
+		command		=> 'auth_info',
+		domain		=> 'example.uk.com',
+		username	=> $handle,
+		password	=> $passwd,
+	);
+
+	my $response = $query->execute;
+
+	if ($response->is_error) {
+		printf("Error: %s\n", $response->error);
+
+	} else {
+		printf("Transfer code: %s\n", $response->auth_code);
+
+	}
+
+This command is used to retrieve the transfer code for a domain name. Those registrars who use our EPP system are required to provide a security code when they initiate a transfer request, which they get from the registrant, who in turn gets it from the losing registrar. Resellers are required to provide this code to registrants upon request.
+
 =head2 Getting Detailed Information About a Handle
 
 This command can be used to retrieve information about a handle I<for which you are the sponsor>. You are the sponsor of a handle if you created it as part of a domain registration or modification process, or it is associated with a domain name for which you are Billing Handle.
@@ -179,7 +210,7 @@ This command can be used to retrieve information about a handle I<for which you
 		printf("Error: %s\n", $response->error);
 
 	} else {
-		foreach my $key (qw(name company address postcode country tel fax email visible)) {
+		foreach my $key (qw(name company address street1 street2 street3 city sp postcode country tel fax email visible)) {
 			printf("%s: %s\n", $key, $response->response($key));
 		}
 
@@ -187,6 +218,8 @@ This command can be used to retrieve information about a handle I<for which you
 
 The C<visible> value is a binary value (1 or 0) that indicates whether this handle's details are show in WHOIS records for domains with which it is associated.
 
+B<Address data:> handles in the CentralNic database follow one of two "styles": old-style handles have flat address data (stored in the C<address> property), but new-style handles have separate properties for street, city and state/province. All handles will have at least a non-empty C<address> property (for new-style handles this is created by flattening the C<street1>, C<street2>... properties). All new-style handles will have a non-empty C<street1> property.
+
 =head1 Creating New Domains and Handles
 
 =head2 Creating a Handle
@@ -207,7 +240,10 @@ To do so, use the C<create_handle> command:
 	$query->set(handle => {
 		name	=> 'John Doe',
 		company	=> 'Example, Inc',
-		address	=> "Example House, Example Street, London",
+		street1	=> 'Example House',
+		street2	=> 'Example Street',
+		city	=> 'London',
+		sp	=> 'England',
 		postcode=> 'EC1 123',
 		country	=> 'UK',
 		phone	=> '+44.8700170900',
@@ -232,7 +268,11 @@ The C<handle> parameter must be a reference to a hash containing the following k
 
 	name
 	company
-	address
+	street1
+	street2
+	street3
+	city
+	sp
 	postcode
 	country
 	phone
@@ -245,6 +285,8 @@ Only C<name>, C<country> and C<email> are mandatory. The C<phone> and C<fax> fie
 
 where C<AA> is the dialling code for the country in question, C<BBB> is the phone number without a local dialling prefix (eg, for the UK, emit the leading zero), and C<CC> is an optional extension.
 
+C<street1>, C<street2> and C<street3> are street address lines, C<city> is the postal city/town and C<sp> is the state, province or county.
+
 The C<country> field must be a valid ISO3166 2-character country code.
 
 The C<visible> parameter controls whether the handle's contact details are shown on whois records for domain names to which it is associated. By default, the value of C<visible> is C<1>. A value of C<0> hides the handle from whois records.
@@ -299,13 +341,11 @@ C<registrant> - the name of the domain's registrant. This is a text string corre
 
 =item 2
 
-C<chandle> - the Client Handle. This may take two values. It can either be a scalar containing the Handle ID of an existing handle, or a reference to a hash with the same format as specified in the "Creating a Handle" section above.
-
-The Client Handle should correspond to your customer's contact details.
+C<chandle> - the Client Handle. The Client Handle should correspond to your customer's contact details.
 
 =item 3
 
-C<thandle> - the Technical Handle. This may take I<three> values. It can be a scalar containing the Handle ID of an existing handle, or "C<chandle>" to set it to be whatever C<chandle> is, or another reference to a hash.
+C<thandle> - the Technical Handle. This may take two values. It can be a scalar containing the Handle ID of an existing handle, or "C<chandle>" to set it to be whatever C<chandle> is.
 
 =item 4
 
@@ -326,6 +366,8 @@ C<period> - the registration period for the domain name. This is an integer numb
 
 The response object for this command has all the usual methods (as documented in L<WWW::CNic::Response>), plus the C<amount()> method, which returns the price in Sterling for the domain.
 
+B<NB>: In addition to specifying a handle ID or "C<chandle>" in the C<chandle> and C<thandle> fields, it is also possible to use an anonymous hash to supply the details of a new handle. However this is a legacy feature for compatibility with older versions of WWW::CNic and its use is not recommended.
+
 =head1 Modifying Domain Names and Handles
 
 =head2 Modifying a Domain Name
@@ -366,37 +408,13 @@ The C<modify> command allows you to add and remove DNS servers and to change the
 
 =item 1
 
-C<thandle> corresponds to a new Technical Handle. The allowed values are the same as those allowed for handle parameters in the C<register> command - either a scalar containing a Handle ID, or an reference to a hash formatted in the way described above.
+C<thandle> corresponds to a new Technical Handle.
 
 =item 2
 
 C<dns> must be an anonymous hash (or a reference to a hash) with two keys: C<add> and C<drop>. Their values are anonymous arrays of DNS hostnames. When dropping DNS servers, you can use the string C<'all'> to indicate that you want to delete all the previously delegated DNS servers (this doesn't affect any servers you might add during the same transaction).
 
-=back
-
-If you create a new Technical Handle, you can use the C<response()> method of the response object to find out its handle ID. For example:
-
-	use WWW::CNic;
-
-	my $query = WWW::CNic->new(
-		use_ssl		=> 1,
-		command		=> 'modify',
-		domain		=> 'example.uk.com',
-		username	=> $handle,
-		password	=> $passwd,
-	);
-
-	$query->set(thandle => $thandle);
-
-	my $response = $query->execute;
-
-	if ($response->is_success) {
-		printf("Added technical handle %s.\n", $response->response('thandle'));
-
-	} else {
-		printf("Error, could not modify domain: %s\n", $response->error);
-
-	}
+B<NB>: In addition to specifying a handle ID in the C<thandle> field, it is also possible to use an anonymous hash to supply the details of a new handle. However this usage is not recommended.
 
 =head2 Modifying a Handle
 
@@ -417,7 +435,10 @@ B<IMPORTANT>: this command B<must not> be used to completely change the contact
 		handle		=> 'H12345',
 		name		=> 'John Doe',
 		company		=> 'Example, Inc',
-		address		=> "Example House, Example Street, London",
+		street1		=> 'Example House',
+		street2		=> 'Example Street',
+		city		=> 'London',
+		sp		=> 'England',
 		postcode	=> 'EC1 123',
 		country		=> 'UK',
 		phone		=> '+44.8700170900',
@@ -440,7 +461,11 @@ You can change the following values:
 
 	name
 	company
-	address
+	street1
+	street2
+	street3
+	city
+	sp
 	postcode
 	country
 	phone
@@ -685,6 +710,46 @@ You can B<start> a transfer process for one or more domain names using the C<sta
 
 	}
 
+=head2 Getting a List of Pending Transfers
+
+	my $query = WWW::CNic->new(
+		use_ssl		=> 1,
+		command		=> 'poll_transfers',
+		username	=> $handle,
+		password	=> $password,
+	);
+
+	my $response = $query->execute;
+
+	if ($response->is_error) {
+		printf("Error: %s\n", $response->error);
+	}
+
+	foreach my $transfer ($response->transfers) {
+		printf("Domain: %s\n", $transfer->{domain});
+
+		if ($transfer->{type} eq 'in') {
+			printf("\tLosing handle: %s (email %s)\n", $transfer->{losing_id}, $transfer->{losing_email});
+
+		} elsif ($transfer->{type} eq 'out') {
+			printf("\tGaining handle: %s (email %s)\n", $transfer->{gaining_id}, $transfer->{gaining_email});
+
+		}
+
+		printf(
+			"\tTransfer initiated on %s\n",
+			scalar(localtime($transfer->{initiated}))
+		);
+
+		printf(
+			"\tTransfer expires on on %s\n",
+			scalar(localtime($transfer->{actiondate}))
+		);
+
+	}
+
+The C<transfers()> method returns an array of hashrefs containing transfer information. If there are no transfers, then the array will be empty.
+
 =head2 Cancelling a Domain Transfer
 
 	my $query = WWW::CNic->new(
@@ -731,6 +796,50 @@ This lets you query the status of a domain name transfer. The returned status is
 
 If there are no transfers for the domain name, or you are not the gaining handle, the server will return an error.
 
+=head2 Approving a Domain Transfer
+
+	my $query = WWW::CNic->new(
+		use_ssl		=> 1,
+		command		=> 'approve_transfer',
+		username	=> $handle,
+		password	=> $password,
+		domain		=> 'example.uk.com',
+	);
+
+	my $response = $query->execute;
+
+	if ($response->is_success) {
+		print "Transfer approved.\n";
+
+	} else {
+		printf("Error: %s\n", $response->error);
+
+	}
+
+If there are no transfers for the domain name, or you are not the losing handle, the server will return an error.
+
+=head2 Rejecting a Domain Transfer
+
+	my $query = WWW::CNic->new(
+		use_ssl		=> 1,
+		command		=> 'reject_transfer',
+		username	=> $handle,
+		password	=> $password,
+		domain		=> 'example.uk.com',
+	);
+
+	my $response = $query->execute;
+
+	if ($response->is_success) {
+		print "Transfer rejected.\n";
+
+	} else {
+		printf("Error: %s\n", $response->error);
+
+	}
+
+If there are no transfers for the domain name, or you are not the losing handle, the server will return an error.
+
 =head1 Account Management
 
 =head2 Getting a List of Upcoming Renewals
@@ -835,7 +944,7 @@ We're always keen to find out about how people are using our Toolkit system, and
 
 =head1 Copyright
 
-This module is (c) 2006 CentralNic Ltd. All rights reserved. This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
+This module is (c) 2007 CentralNic Ltd. All rights reserved. This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
 =head1 See Also
 
